/***************************************************************************
                        blocklist.h  -  list of blocks
                       --------------------------------
    begin                : Sat Jul 29 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BLOCKLIST_H
#define BLOCKLIST_H

#include <vector>
#include <string>

#include "soundblock.h"
#include "mp3block.h"
#include "sounddevice.h"
#include "dialogprogress.h"
#include "dialogtageditor.h"

#include <sigc++/signal_system.h>
#include <gdk--/pixmap.h>
#include <gdk--/gc.h>

using namespace std;

/**Base class with array of sound blocks
  *@author Ladislav Slezk
  */

class BlockList : public SigC::Object {
	
  public:

  	BlockList();
  	BlockList(const BlockList &src);
  	~BlockList();
  	
    void add_block(SoundBlock *);        // add block to list
    void clear();
    void display_blocks(Gdk_Pixmap *, Gdk_GC);      // draw waveform on pixmap
    int total_ms();                      // total playing time
//    int playedMS() {return totalms;}
    bool is_playing();                   // a block is playing
    bool is_paused();                    // a block is playing
    bool is_changed() {return chg;}
    int num_blocks();                    // number of blocks in list
    SoundBlock *block_at(int);           // return block
    void remove_block(int);              // remove block from list
    bool block_to_end(int);               // move block to end
    bool block_to_begin(int);             // move block to start

    void select(int from_ms, int to_ms);    // select part of block list -- paramters in miliseconds
    int select_block(int offset_ms);        // select one block which is in offset_ms, return offset of selected block (in ms)
    void delete_block(int offset_ms);       // delete block at offset offset_ms
    void resize_block(int offset_ms);
    void cut(int from_ms, int to_ms);       // delete from block list
    bool cutting;                           // cut is in progress
    void paste(int offset_ms, vector<SoundBlock *> &b);
    void paste_before(int offset_ms, vector<SoundBlock *> &b);
    void paste_after(int offset_ms, vector<SoundBlock *> &b);
    vector<SoundBlock *> get_vector_copy();

    int index_at_offset(int offset_ms);
    SoundBlock *block_at_offset(int offset_ms);

    string get_file_name(){return fileName;}

    void play_list(SoundDevice *sd, string mpg123_path, int from_ms = -1, int to_ms = -1);       // play list

    void stop_list();
    void pause_list();
    void unpause_list();
		
		bool save_BlockList_to_file(string fname, bool ask_rewrite = true);
		bool load_BlockList_from_file(string);
		
		bool export_list_WAV(string file, int ch, int f, int bps, DialogProgress *dp, string export_mpg123_path, string export_sox_path);
		
		bool export_list_MP3(string file, string encoder_name, string encoder_path, int bitrate, int frequency,
		                     MP3block::ChannelMode channel_mode, bool crc, bool priv, bool original,
		                     bool copyright, vector<string> VBR_options, vector<string> encoder_options,
		                     bool always_process, bool strict_equal, DialogTagEditor *id3, DialogProgress *dp);
		
	private:		// slots
	
		void scanned();
		
	public:  	// signals
	
		SigC::Signal0<void> play_done;
		SigC::Signal1<void, int> play_block_done;
		SigC::Signal0<void> scan_done;
		SigC::Signal0<void> export_done;
		SigC::Signal0<void> block_list_changed;
  	
  protected:

    typedef enum {EXPORT_WAV, EXPORT_MP3} export_format;

    export_format export_type;
  	
    vector<SoundBlock *> blocks;     // array of blocks
    bool playing;
    bool paused;
    bool chg;                        // list changed
    SoundBlock *actual;
    int actualIndex;
    SoundDevice *Sdevice;            // sound device
		string fileName;                 // file name
		int totalms, totaldone;
		DialogProgress *export_progress;
		
//		void blockPlayedMiliSecs(int);
		void sound_block_changed();      // sound block callback
		void block_play_done();
		void block_export_done();

    int play_to_ms, play_from_ms;
		int firstblock, lastblock;
		int ms1, ms2;

    ofstream export_stream;
    int export_channels, export_freq, export_bps;

		string export_encoder_name, export_encoder_path, export_mpg123_path, export_sox_path;
		
		int export_bitrate;
		MP3block::ChannelMode export_channel_mode;
    bool export_crc, export_priv, export_original, export_copyright;
    vector<string> export_VBR_options, export_encoder_options;
    bool export_strict_equal, export_always_process;
    DialogTagEditor *export_id3;

    string mpg123_decoder;

};

#endif
