/***************************************************************************
          dialogexportmp3.cpp  -  Dialog for setting MP3 parameters
         -----------------------------------------------------------
    begin                : Fri Mar 30 2001
    copyright            : (C) 2001 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dialogexportmp3.h"

#include <libgnome/gnome-i18n.h>
#include <gnome--/entry.h>


DialogExportMP3::DialogExportMP3() : Gtk::Window(GTK_WINDOW_DIALOG) {
  set_modal(true);
  set_title(_("Export as MP3 file"));
  set_policy(false, false, false);    // set fixed size of window

  string history;
  fentry = manage(new Gnome::FileEntry(history, _("Select MP3 file")));
  fentry->set_modal(true);
  fentry->set_border_width(5);
  fentry->set_default_path("*.mp3");

  Gnome::Entry *gentry = dynamic_cast<Gnome::Entry *>(fentry->gnome_entry());
  gentry->set_history_id("MP3_export_history");
  gentry->load_history();

  Gtk::Frame *frame_fname = manage(new Gtk::Frame(_("MP3 File Name")));
  frame_fname->set_border_width(3);
  frame_fname->add(*fentry);

  Gtk::Label *encoder_label = manage(new Gtk::Label(_("Use MPEG encoder")));
  Gtk::OptionMenu *encoder_option_menu = manage(new Gtk::OptionMenu);
  Gtk::Menu *encoder_menu = manage(new Gtk::Menu);

  Gtk::MenuItem *enc_lame = manage(new Gtk::MenuItem("lame"));
  enc_lame->activate.connect(bind(slot(this, &DialogExportMP3::encoder_changed), string("lame")));
  encoder_menu->append(*enc_lame);

//  Gtk::MenuItem *enc_blade = manage(new Gtk::MenuItem("bladeenc"));
//  enc_blade->activate.connect(bind(slot(this, &DialogExportMP3::encoder_changed), string("bladeenc")));
//  enc_blade->set_sensitive(false);
//  encoder_menu->append(*enc_blade);

  encoder_menu->set_active(0);
  encoder_menu->show_all();
  encoder_option_menu->set_menu(encoder_menu);

  Gtk::Button *encoder_button = manage(new Gtk::Button(_("Options ...")));
  encoder_button->clicked.connect(slot(this, &DialogExportMP3::show_lame_options));
  Gtk::HBox *encoder_hbox = manage(new Gtk::HBox(false, 5));
  encoder_hbox->set_border_width(5);
  encoder_hbox->pack_start(*encoder_label, false, false);
  encoder_hbox->pack_end(*encoder_button, false, false);
  encoder_hbox->pack_end(*encoder_option_menu, false, false);
  Gtk::Frame *frame_encoder = manage(new Gtk::Frame(_("MPEG encoder")));
  frame_encoder->add(*encoder_hbox);
  frame_encoder->set_border_width(3);


  Gtk::Label *stream_bitrate_label = manage(new Gtk::Label(_("Total bitrate")));
  Gtk::OptionMenu *stream_bitrate_optionmenu = manage(new Gtk::OptionMenu);
  Gtk::Menu *bitrate_menu = manage(new Gtk::Menu);

  br8 = manage(new Gtk::MenuItem("8 kb/s"));
  br8->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 8));
  bitrate_menu->append(*br8);

  br16 = manage(new Gtk::MenuItem("16 kb/s"));
  br16->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 16));
  bitrate_menu->append(*br16);

  br24 = manage(new Gtk::MenuItem("24 kb/s"));
  br24->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 24));
  bitrate_menu->append(*br24);

  br32 = manage(new Gtk::MenuItem("32 kb/s"));
  br32->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 32));
  bitrate_menu->append(*br32);

  br40 = manage(new Gtk::MenuItem("40 kb/s"));
  br40->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 40));
  bitrate_menu->append(*br40);

  br48 = manage(new Gtk::MenuItem("48 kb/s"));
  br48->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 48));
  bitrate_menu->append(*br48);

  br56 = manage(new Gtk::MenuItem("56 kb/s"));
  br56->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 56));
  bitrate_menu->append(*br56);

  br64 = manage(new Gtk::MenuItem("64 kb/s"));
  br64->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 64));
  bitrate_menu->append(*br64);

  br80 = manage(new Gtk::MenuItem("80 kb/s"));
  br80->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 80));
  bitrate_menu->append(*br80);

  br96 = manage(new Gtk::MenuItem("96 kb/s"));
  br96->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 96));
  bitrate_menu->append(*br96);

  br112 = manage(new Gtk::MenuItem("112 kb/s"));
  br112->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 112));
  bitrate_menu->append(*br112);

  br128 = manage(new Gtk::MenuItem("128 kb/s"));
  br128->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 128));
  bitrate_menu->append(*br128);

  br144 = manage(new Gtk::MenuItem("144 kb/s"));
  br144->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 144));
  bitrate_menu->append(*br144);

  br160 = manage(new Gtk::MenuItem("160 kb/s"));
  br160->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 160));
  bitrate_menu->append(*br160);

  br192 = manage(new Gtk::MenuItem("192 kb/s"));
  br192->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 192));
  bitrate_menu->append(*br192);

  br224 = manage(new Gtk::MenuItem("224 kb/s"));
  br224->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 224));
  bitrate_menu->append(*br224);

  br256 = manage(new Gtk::MenuItem("256 kb/s"));
  br256->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 256));
  bitrate_menu->append(*br256);

  br320 = manage(new Gtk::MenuItem("320 kb/s"));
  br320->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), 320));
  bitrate_menu->append(*br320);

  br_vbr = manage(new Gtk::MenuItem(_("Variable bitrate")));
  br_vbr->activate.connect(bind(slot(this, &DialogExportMP3::bitrate_changed), -1));
  bitrate_menu->append(*br_vbr);

  bitrate_menu->set_active(11);
  bitrate_menu->show_all();
  stream_bitrate_optionmenu->set_menu(bitrate_menu);

  stream_bitrate_button = manage(new Gtk::Button(_("VBR options ...")));
  stream_bitrate_button->clicked.connect(slot(this, &DialogExportMP3::show_vbr_options));
  stream_bitrate_button->set_sensitive(false);
  Gtk::HBox *stream_bitrate_hbox = manage(new Gtk::HBox(false, 5));
  stream_bitrate_hbox->set_border_width(5);
  stream_bitrate_hbox->pack_start(*stream_bitrate_label, false, false);
  stream_bitrate_hbox->pack_end(*stream_bitrate_button, false, false);
  stream_bitrate_hbox->pack_end(*stream_bitrate_optionmenu, false, false);

  Gtk::Label *stream_freq_label = manage(new Gtk::Label(_("Sampling frequency")));
  Gtk::OptionMenu *stream_freq_optionmenu = manage(new Gtk::OptionMenu);
  Gtk::Menu *stream_freq_menu = manage(new Gtk::Menu);

  f16000 = manage(new Gtk::MenuItem(_("16000 Hz")));
  f16000->activate.connect(bind(slot(this, &DialogExportMP3::freq_changed), 16000));
  stream_freq_menu->append(*f16000);

  f22050 = manage(new Gtk::MenuItem(_("22050 Hz")));
  f22050->activate.connect(bind(slot(this, &DialogExportMP3::freq_changed), 22050));
  stream_freq_menu->append(*f22050);

  f24000 = manage(new Gtk::MenuItem(_("24000 Hz")));
  f24000->activate.connect(bind(slot(this, &DialogExportMP3::freq_changed), 24000));
  stream_freq_menu->append(*f24000);

  f32000 = manage(new Gtk::MenuItem(_("32000 Hz")));
  f32000->activate.connect(bind(slot(this, &DialogExportMP3::freq_changed), 32000));
  stream_freq_menu->append(*f32000);

  f44100 = manage(new Gtk::MenuItem(_("44100 Hz")));
  f44100->activate.connect(bind(slot(this, &DialogExportMP3::freq_changed), 44100));
  stream_freq_menu->append(*f44100);

  f48000 = manage(new Gtk::MenuItem(_("48000 Hz")));
  f48000->activate.connect(bind(slot(this, &DialogExportMP3::freq_changed), 48000));
  stream_freq_menu->append(*f48000);

  stream_freq_menu->set_active(4);
  stream_freq_menu->show_all();
  stream_freq_optionmenu->set_menu(stream_freq_menu);
  Gtk::HBox *stream_freq_hbox = manage(new Gtk::HBox(false, 5));
  stream_freq_hbox->set_border_width(5);
  stream_freq_hbox->pack_start(*stream_freq_label, false, false);
  stream_freq_hbox->pack_end(*stream_freq_optionmenu, false, false);

  Gtk::Label *stream_mode_label = manage(new Gtk::Label(_("Channel mode")));
  Gtk::OptionMenu *stream_mode_optionmenu = manage(new Gtk::OptionMenu);
  Gtk::Menu *stream_mode_menu = manage(new Gtk::Menu);
  Gtk::Menu_Helpers::MenuList &stream_mode_menu_list = stream_mode_menu->items();
  stream_mode_menu_list.push_back(Gtk::Menu_Helpers::MenuElem(_("Joint stereo"), bind(slot(this, &DialogExportMP3::mode_changed), MP3block::MODE_JOINT_STEREO)));
  stream_mode_menu_list.push_back(Gtk::Menu_Helpers::MenuElem(_("Stereo"), bind(slot(this, &DialogExportMP3::mode_changed), MP3block::MODE_STEREO)));
  stream_mode_menu_list.push_back(Gtk::Menu_Helpers::MenuElem(_("Mono"), bind(slot(this, &DialogExportMP3::mode_changed), MP3block::MODE_MONO)));
  stream_mode_menu->set_active(1);
  stream_mode_optionmenu->set_menu(stream_mode_menu);
  Gtk::HBox *stream_mode_hbox = manage(new Gtk::HBox(false, 5));
  stream_mode_hbox->set_border_width(5);
  stream_mode_hbox->pack_start(*stream_mode_label, false, false);
  stream_mode_hbox->pack_end(*stream_mode_optionmenu, false, false);

  stream_private = manage(new Gtk::CheckButton(_("Private"), 0.0));
  stream_private->set_sensitive(false);
  stream_crc = manage(new Gtk::CheckButton(_("Use CRC"), 0.0));
  stream_original = manage(new Gtk::CheckButton(_("Original"), 0.0));
  stream_copyright = manage(new Gtk::CheckButton(_("Copyright"), 0.0));
  Gtk::Table *stream_table = manage(new Gtk::Table(2, 2, true));
  stream_table->set_border_width(3);
  stream_table->set_row_spacings(3);
  stream_table->set_col_spacings(3);
  stream_table->attach(*stream_private, 1, 2, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
  stream_table->attach(*stream_crc, 0, 1, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
  stream_table->attach(*stream_original, 0, 1, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
  stream_table->attach(*stream_copyright, 1, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);

  Gtk::VBox *stream_vbox = manage(new Gtk::VBox(false, 0));
  stream_vbox->pack_start(*stream_bitrate_hbox);
  stream_vbox->pack_start(*stream_freq_hbox);
  stream_vbox->pack_start(*stream_mode_hbox);
  stream_vbox->pack_start(*stream_table);
  Gtk::Frame *frame_stream = manage(new Gtk::Frame(_("MPEG stream options")));
  frame_stream->set_border_width(3);
  frame_stream->add(*stream_vbox);

  tag_checkbutton = manage(new Gtk::CheckButton(_("Use ID3 tag"), 0.0));
  tag_checkbutton->clicked.connect(slot(this, &DialogExportMP3::state_edit_tag));
  tag_button = manage(new Gtk::Button(_("Edit ID3 tag ...")));
  tag_button->clicked.connect(slot(this, &DialogExportMP3::show_tag_editor));
  tag_button->set_sensitive(false);
  Gtk::HBox *tag_hbox = manage(new Gtk::HBox(false, 5));
  tag_hbox->set_border_width(5);
  tag_hbox->pack_start(*tag_checkbutton, true, true);
  tag_hbox->pack_start(*tag_button, false, false);

  Gtk::Frame *frame_tag = manage(new Gtk::Frame(_("ID3 tag")));
  frame_tag->set_border_width(3);
  frame_tag->add(*tag_hbox);


  Gnome::StockButton *button_ok = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_OK));
  button_ok->clicked.connect(bind(slot(this, &DialogExportMP3::button_pressed), true));
  Gnome::StockButton *button_cancel = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_CANCEL));
  button_cancel->clicked.connect(bind(slot(this, &DialogExportMP3::button_pressed), false));

  Gtk::HButtonBox *hbuttonbox = manage(new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD, 5));
  hbuttonbox->add(*button_ok);
  hbuttonbox->add(*button_cancel);
  hbuttonbox->set_border_width(5);


  Gtk::VBox *vbox = manage(new Gtk::VBox(false, 3));
  vbox->pack_start(*frame_fname, false, false);
  vbox->pack_start(*frame_encoder, false, false);
  vbox->pack_start(*frame_stream, false, false);
  vbox->pack_start(*frame_tag, false, false);
  vbox->pack_start(*hbuttonbox, false, false);
  vbox->set_border_width(5);
  add(*vbox);

  freq_changed(44100);     // set default 44100 Hz
  freq = 44100;
  bitrate_changed(128);    // set default 128 kb/s
  bitrate = 128;
  mode = MP3block::MODE_STEREO;     // default mode is stereo
  encoder_name = "lame";

  tag_editor = manage(new DialogTagEditor);
  tag_editor->destroy.connect(slot(this, &DialogExportMP3::tag_editor_destroyed));
  vbr_options = manage(new DialogVBRoptions);
  vbr_options->set_sampling_frequency(freq);
  vbr_options->destroy.connect(slot(this, &DialogExportMP3::vbr_dialog_destroyed));
  lame_options = manage(new DialogLameOptions);
  lame_options->destroy.connect(slot(this, &DialogExportMP3::lame_dialog_destroyed));

  show_all();
}

DialogExportMP3::~DialogExportMP3() {

}

void DialogExportMP3::state_edit_tag() {
  tag_button->set_sensitive(tag_checkbutton->get_active());
}

void DialogExportMP3::freq_changed(int f) {
#ifdef DEBUG_OUTPUT
  cerr << "freq = " << f << endl;
#endif

  switch(f) {
    case(32000):
    case(44100):
    case(48000):
      br8->set_sensitive(false);
      br16->set_sensitive(false);
      br24->set_sensitive(false);
      br32->set_sensitive(true);
      br40->set_sensitive(true);
      br48->set_sensitive(true);
      br56->set_sensitive(true);
      br64->set_sensitive(true);
      br80->set_sensitive(true);
      br96->set_sensitive(true);
      br112->set_sensitive(true);
      br128->set_sensitive(true);
      br144->set_sensitive(false);
      br160->set_sensitive(true);
      br192->set_sensitive(true);
      br224->set_sensitive(true);
      br256->set_sensitive(true);
      br320->set_sensitive(true);
      br_vbr->set_sensitive(true);
      break;

    case(16000):
    case(22050):
    case(24000):
      br8->set_sensitive(true);
      br16->set_sensitive(true);
      br24->set_sensitive(true);
      br32->set_sensitive(true);
      br40->set_sensitive(true);
      br48->set_sensitive(true);
      br56->set_sensitive(true);
      br64->set_sensitive(true);
      br80->set_sensitive(true);
      br96->set_sensitive(true);
      br112->set_sensitive(true);
      br128->set_sensitive(true);
      br144->set_sensitive(true);
      br160->set_sensitive(true);
      br192->set_sensitive(false);
      br224->set_sensitive(false);
      br256->set_sensitive(false);
      br320->set_sensitive(false);
      br_vbr->set_sensitive(true);
      break;
  }

  freq = f;
}

void DialogExportMP3::bitrate_changed(int br) {
#ifdef DEBUG_OUTPUT
  cerr << "bitrate = " << br << endl;
#endif

  switch(br) {
    case(32):case(48):case(56):       // all freqencies allowed
    case(64):case(80):case(96):
    case(112):case(128):case(160):case(-1):
      f16000->set_sensitive(true);
      f22050->set_sensitive(true);
      f24000->set_sensitive(true);
      f32000->set_sensitive(true);
      f44100->set_sensitive(true);
      f48000->set_sensitive(true);
      break;

    case(8):case(16):case(24):case(144):    // enable only higher freqencies
      f16000->set_sensitive(true);
      f22050->set_sensitive(true);
      f24000->set_sensitive(true);
      f32000->set_sensitive(false);
      f44100->set_sensitive(false);
      f48000->set_sensitive(false);
      break;

    case(192):case(224):case(256):case(320):    // enable only lower freqencies
      f16000->set_sensitive(false);
      f22050->set_sensitive(false);
      f24000->set_sensitive(false);
      f32000->set_sensitive(true);
      f44100->set_sensitive(true);
      f48000->set_sensitive(true);
      break;
  }

  stream_bitrate_button->set_sensitive(br < 0);  // enable VBR option dialog
  bitrate = br;
}

void DialogExportMP3::mode_changed(MP3block::ChannelMode m) {
  mode = m;
}

void DialogExportMP3::button_pressed(bool ok) {
  Gnome::Entry *gentry = dynamic_cast<Gnome::Entry *>(fentry->gnome_entry());
  if (gentry) gentry->save_history();

  selection_done.emit(ok);
  hide();
}

void DialogExportMP3::show_tag_editor() {
  if (tag_editor) {
    tag_editor->show_all();
  }
  else {
    tag_editor = manage(new DialogTagEditor);
    tag_editor->destroy.connect(slot(this, &DialogExportMP3::tag_editor_destroyed));
  }
}

void DialogExportMP3::show_lame_options() {
  if (lame_options) {
    lame_options->show_all();
  }
  else {
    lame_options = manage(new DialogLameOptions);
    lame_options->destroy.connect(slot(this, &DialogExportMP3::lame_dialog_destroyed));
  }
}

void DialogExportMP3::show_vbr_options() {
  if (vbr_options) {
    vbr_options->set_sampling_frequency(freq);
    vbr_options->show_all();
  }
  else {
    vbr_options = manage(new DialogVBRoptions);
    vbr_options->set_sampling_frequency(freq);
    vbr_options->destroy.connect(slot(this, &DialogExportMP3::vbr_dialog_destroyed));
  }
}


void DialogExportMP3::tag_editor_destroyed() {
#ifdef DEBUG_OUTPUT
  cerr << "tag editor destoyed" << endl;
#endif

  tag_editor = 0;
}

void DialogExportMP3::vbr_dialog_destroyed() {
#ifdef DEBUG_OUTPUT
  cerr << "VBR dialog destroyed" << endl;
#endif

  vbr_options = 0;
}

void DialogExportMP3::lame_dialog_destroyed() {
#ifdef DEBUG_OUTPUT
  cerr << "LAME options dialog destroyed" << endl;
#endif

  lame_options = 0;
}

int DialogExportMP3::delete_event_impl(GdkEventAny* e) {
  hide();

  return true;            // disable destroying window by WM
}

string DialogExportMP3::get_filename() {
  // method gnome_entry returns Gtk::Widget *, dynamic cast is required
  Gnome::Entry *gentry = dynamic_cast<Gnome::Entry *>(fentry->gnome_entry());

  if (gentry)
    return gentry->get_entry()->get_text();
  else
    return string("");
}

vector<string> DialogExportMP3::get_vbr_options() {
  if (vbr_options)
    return vbr_options->get_vbr_options();
  else {
    vector<string> tmp;
    return tmp;
  }
}

vector<string> DialogExportMP3::get_encoder_options() {
  if (lame_options)
    return lame_options->get_lame_parameters();
  else {
    vector<string> tmp;
    return tmp;
  }
}

void DialogExportMP3::encoder_changed(string name) {
  encoder_name = name;
}

DialogTagEditor *DialogExportMP3::get_tag_editor() {
  if (tag_checkbutton->get_active())
    return tag_editor;
  else
    return 0;
}
