/***************************************************************************
      dialogvbroptions.cpp  -  Dialog for setting VBR encoding options
     ------------------------------------------------------------------
    begin                : Sat Mar 31 2001
    copyright            : (C) 2001 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "dialogvbroptions.h"

#include <libgnome/gnome-i18n.h>
#include <gnome--/dialog.h>

#include <strstream>


DialogVBRoptions::DialogVBRoptions() : Gtk::Window(GTK_WINDOW_DIALOG) {
  set_modal(true);
  set_title(_("Variable bitrate options"));
  set_policy(false, false, false);    // set fixed size of window

  Gtk::OptionMenu *quality_optionmenu = manage(new Gtk::OptionMenu);
  Gtk::Menu *quality_menu = manage(new Gtk::Menu);

  Gtk::MenuItem *q0 = manage(new Gtk::MenuItem(_("0 - high quality")));
  q0->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 0));
  quality_menu->append(*q0);

  Gtk::MenuItem *q1 = manage(new Gtk::MenuItem(_("1")));
  q1->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 1));
  quality_menu->append(*q1);

  Gtk::MenuItem *q2 = manage(new Gtk::MenuItem(_("2")));
  q2->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 2));
  quality_menu->append(*q2);

  Gtk::MenuItem *q3 = manage(new Gtk::MenuItem(_("3")));
  q3->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 3));
  quality_menu->append(*q3);

  Gtk::MenuItem *q4 = manage(new Gtk::MenuItem(_("4 - default")));
  q4->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 4));
  quality_menu->append(*q4);

  Gtk::MenuItem *q5 = manage(new Gtk::MenuItem(_("5")));
  q5->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 5));
  quality_menu->append(*q5);

  Gtk::MenuItem *q6 = manage(new Gtk::MenuItem(_("6")));
  q6->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 6));
  quality_menu->append(*q6);

  Gtk::MenuItem *q7 = manage(new Gtk::MenuItem(_("7")));
  q7->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 7));
  quality_menu->append(*q7);

  Gtk::MenuItem *q8 = manage(new Gtk::MenuItem(_("8")));
  q8->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 8));
  quality_menu->append(*q8);

  Gtk::MenuItem *q9 = manage(new Gtk::MenuItem(_("9 - low quality")));
  q9->activate.connect(bind(slot(this, &DialogVBRoptions::quality_changed), 9));
  quality_menu->append(*q9);

  quality_menu->set_active(4);
  quality_menu->show_all();
  quality_optionmenu->set_menu(quality_menu);



  Gtk::Label *label_quality = manage(new Gtk::Label(_("Quality setting for VBR")));
  Gtk::HBox *hbox_quality = manage(new Gtk::HBox(false, 5));
  hbox_quality->pack_start(*label_quality, false, false);
  hbox_quality->pack_end(*quality_optionmenu, false, false);
  hbox_quality->set_border_width(3);

  min_bitrate_optionmenu = manage(new Gtk::OptionMenu);
  min_bitrate_menu = manage(new Gtk::Menu);

  min_br8 = manage(new Gtk::MenuItem("8 kb/s"));
  min_br8->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 8));
  min_bitrate_menu->append(*min_br8);

  min_br16 = manage(new Gtk::MenuItem("16 kb/s"));
  min_br16->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 16));
  min_bitrate_menu->append(*min_br16);

  min_br24 = manage(new Gtk::MenuItem("24 kb/s"));
  min_br24->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 24));
  min_bitrate_menu->append(*min_br24);

  min_br32 = manage(new Gtk::MenuItem("32 kb/s"));
  min_br32->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 32));
  min_bitrate_menu->append(*min_br32);

  min_br40 = manage(new Gtk::MenuItem("40 kb/s"));
  min_br40->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 40));
  min_bitrate_menu->append(*min_br40);

  min_br48 = manage(new Gtk::MenuItem("48 kb/s"));
  min_br48->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 48));
  min_bitrate_menu->append(*min_br48);

  min_br56 = manage(new Gtk::MenuItem("56 kb/s"));
  min_br56->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 56));
  min_bitrate_menu->append(*min_br56);

  min_br64 = manage(new Gtk::MenuItem("64 kb/s"));
  min_br64->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 64));
  min_bitrate_menu->append(*min_br64);

  min_br80 = manage(new Gtk::MenuItem("80 kb/s"));
  min_br80->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 80));
  min_bitrate_menu->append(*min_br80);

  min_br96 = manage(new Gtk::MenuItem("96 kb/s"));
  min_br96->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 96));
  min_bitrate_menu->append(*min_br96);

  min_br112 = manage(new Gtk::MenuItem("112 kb/s"));
  min_br112->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 112));
  min_bitrate_menu->append(*min_br112);

  min_br128 = manage(new Gtk::MenuItem("128 kb/s"));
  min_br128->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 128));
  min_bitrate_menu->append(*min_br128);

  min_br144 = manage(new Gtk::MenuItem("144 kb/s"));
  min_br144->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 144));
  min_bitrate_menu->append(*min_br144);

  min_br160 = manage(new Gtk::MenuItem("160 kb/s"));
  min_br160->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 160));
  min_bitrate_menu->append(*min_br160);

  min_br192 = manage(new Gtk::MenuItem("192 kb/s"));
  min_br192->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 192));
  min_bitrate_menu->append(*min_br192);

  min_br224 = manage(new Gtk::MenuItem("224 kb/s"));
  min_br224->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 224));
  min_bitrate_menu->append(*min_br224);

  min_br256 = manage(new Gtk::MenuItem("256 kb/s"));
  min_br256->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 256));
  min_bitrate_menu->append(*min_br256);

  min_br320 = manage(new Gtk::MenuItem("320 kb/s"));
  min_br320->activate.connect(bind(slot(this, &DialogVBRoptions::min_bitrate_changed), 320));
  min_bitrate_menu->append(*min_br320);

  min_bitrate_menu->set_active(3);
  min_bitrate_menu->show_all();
  min_bitrate_optionmenu->set_menu(min_bitrate_menu);
  min_bitrate_optionmenu->set_sensitive(false);

  min_br = manage(new Gtk::CheckButton(_("Minimum allowed bitrate"), 0.0));
  min_br->clicked.connect(slot(this, &DialogVBRoptions::min_bitrate_sensitive));

  Gtk::HBox *hbox_min = manage(new Gtk::HBox(false, 3));
  hbox_min->pack_start(*min_br, false, false);
  hbox_min->pack_end(*min_bitrate_optionmenu, false, false);
  hbox_min->set_border_width(3);

/////////
  max_bitrate_optionmenu = manage(new Gtk::OptionMenu);
  max_bitrate_menu = manage(new Gtk::Menu);

  max_br8 = manage(new Gtk::MenuItem("8 kb/s"));
  max_br8->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 8));
  max_bitrate_menu->append(*max_br8);

  max_br16 = manage(new Gtk::MenuItem("16 kb/s"));
  max_br16->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 16));
  max_bitrate_menu->append(*max_br16);

  max_br24 = manage(new Gtk::MenuItem("24 kb/s"));
  max_br24->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 24));
  max_bitrate_menu->append(*max_br24);

  max_br32 = manage(new Gtk::MenuItem("32 kb/s"));
  max_br32->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 32));
  max_bitrate_menu->append(*max_br32);

  max_br40 = manage(new Gtk::MenuItem("40 kb/s"));
  max_br40->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 40));
  max_bitrate_menu->append(*max_br40);

  max_br48 = manage(new Gtk::MenuItem("48 kb/s"));
  max_br48->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 48));
  max_bitrate_menu->append(*max_br48);

  max_br56 = manage(new Gtk::MenuItem("56 kb/s"));
  max_br56->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 56));
  max_bitrate_menu->append(*max_br56);

  max_br64 = manage(new Gtk::MenuItem("64 kb/s"));
  max_br64->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 64));
  max_bitrate_menu->append(*max_br64);

  max_br80 = manage(new Gtk::MenuItem("80 kb/s"));
  max_br80->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 80));
  max_bitrate_menu->append(*max_br80);

  max_br96 = manage(new Gtk::MenuItem("96 kb/s"));
  max_br96->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 96));
  max_bitrate_menu->append(*max_br96);

  max_br112 = manage(new Gtk::MenuItem("112 kb/s"));
  max_br112->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 112));
  max_bitrate_menu->append(*max_br112);

  max_br128 = manage(new Gtk::MenuItem("128 kb/s"));
  max_br128->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 128));
  max_bitrate_menu->append(*max_br128);

  max_br144 = manage(new Gtk::MenuItem("144 kb/s"));
  max_br144->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 144));
  max_bitrate_menu->append(*max_br144);

  max_br160 = manage(new Gtk::MenuItem("160 kb/s"));
  max_br160->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 160));
  max_bitrate_menu->append(*max_br160);

  max_br192 = manage(new Gtk::MenuItem("192 kb/s"));
  max_br192->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 192));
  max_bitrate_menu->append(*max_br192);

  max_br224 = manage(new Gtk::MenuItem("224 kb/s"));
  max_br224->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 224));
  max_bitrate_menu->append(*max_br224);

  max_br256 = manage(new Gtk::MenuItem("256 kb/s"));
  max_br256->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 256));
  max_bitrate_menu->append(*max_br256);

  max_br320 = manage(new Gtk::MenuItem("320 kb/s"));
  max_br320->activate.connect(bind(slot(this, &DialogVBRoptions::max_bitrate_changed), 320));
  max_bitrate_menu->append(*max_br320);

  max_bitrate_menu->set_active(11);
  max_bitrate_menu->show_all();
  max_bitrate_optionmenu->set_menu(max_bitrate_menu);
  max_bitrate_optionmenu->set_sensitive(false);


  max_br = manage(new Gtk::CheckButton(_("Maximum allowed bitrate"), 0.0));
  max_br->clicked.connect(slot(this, &DialogVBRoptions::max_bitrate_sensitive));
  Gtk::HBox *hbox_max = manage(new Gtk::HBox(false, 3));
  hbox_max->pack_start(*max_br, false, false);
  hbox_max->pack_end(*max_bitrate_optionmenu, false, false);
  hbox_max->set_border_width(3);

  vbr_info = manage(new Gtk::CheckButton(_("Disable Xing VBR informational tag"), 0.0));
  vbr_info->set_active(true);
  vbr_info->set_border_width(3);

  Gtk::VBox *vbox_options = manage(new Gtk::VBox(false, 3));
  vbox_options->set_border_width(3);
  vbox_options->pack_start(*hbox_quality);
  vbox_options->pack_start(*hbox_min);
  vbox_options->pack_start(*hbox_max);
  vbox_options->pack_start(*vbr_info);

  Gtk::Frame *frame = manage(new Gtk::Frame(_("Stream options")));
  frame->set_border_width(3);
  frame->add(*vbox_options);


  Gnome::StockButton *button_ok = manage(new Gnome::StockButton(GNOME_STOCK_BUTTON_OK));
  button_ok->clicked.connect(slot(this, &DialogVBRoptions::button_pressed));

  Gtk::HButtonBox *hbuttonbox = manage(new Gtk::HButtonBox(GTK_BUTTONBOX_SPREAD, 5));
  hbuttonbox->add(*button_ok);
  hbuttonbox->set_border_width(5);

  Gtk::VBox *vbox = manage(new Gtk::VBox(false, 3));
  vbox->pack_start(*frame, false, false);
  vbox->pack_start(*hbuttonbox, false, false);
  vbox->set_border_width(5);
  add(*vbox);

  // default values
  min_bitrate = 32;
  max_bitrate = 128;
  quality = 4;

}

DialogVBRoptions::~DialogVBRoptions() {
}

void DialogVBRoptions::min_bitrate_changed(int br) {
#ifdef DEBUG_OUTPUT
  cerr << "min bitrate " << br << endl;
#endif

  min_bitrate = br;
}

void DialogVBRoptions::max_bitrate_changed(int br) {
#ifdef DEBUG_OUTPUT
  cerr << "max bitrate " << br << endl;
#endif

  max_bitrate = br;
}

void DialogVBRoptions::min_bitrate_sensitive() {
  min_bitrate_optionmenu->set_sensitive(min_br->get_active());
}

void DialogVBRoptions::max_bitrate_sensitive() {
  max_bitrate_optionmenu->set_sensitive(max_br->get_active());
}

void DialogVBRoptions::quality_changed(int q) {
  quality = q;
}

void DialogVBRoptions::set_sampling_frequency(int f) {
  switch(f) {
    case(32000):
    case(44100):
    case(48000):
      if (min_bitrate <= 24) {
        min_bitrate = 32;
        min_bitrate_optionmenu->set_history(3);
      }

      if (min_bitrate == 144) {
        min_bitrate = 128;
        min_bitrate_optionmenu->set_history(11);
      }

      if (max_bitrate <= 24) {
        max_bitrate = 32;
        max_bitrate_optionmenu->set_history(3);
      }

      if (max_bitrate == 144) {
        max_bitrate = 128;
        max_bitrate_optionmenu->set_history(11);
      }

      min_br8->set_sensitive(false);
      min_br16->set_sensitive(false);
      min_br24->set_sensitive(false);
      min_br32->set_sensitive(true);
      min_br40->set_sensitive(true);
      min_br48->set_sensitive(true);
      min_br56->set_sensitive(true);
      min_br64->set_sensitive(true);
      min_br80->set_sensitive(true);
      min_br96->set_sensitive(true);
      min_br112->set_sensitive(true);
      min_br128->set_sensitive(true);
      min_br144->set_sensitive(false);
      min_br160->set_sensitive(true);
      min_br192->set_sensitive(true);
      min_br224->set_sensitive(true);
      min_br256->set_sensitive(true);
      min_br320->set_sensitive(true);

      max_br8->set_sensitive(false);
      max_br16->set_sensitive(false);
      max_br24->set_sensitive(false);
      max_br32->set_sensitive(true);
      max_br40->set_sensitive(true);
      max_br48->set_sensitive(true);
      max_br56->set_sensitive(true);
      max_br64->set_sensitive(true);
      max_br80->set_sensitive(true);
      max_br96->set_sensitive(true);
      max_br112->set_sensitive(true);
      max_br128->set_sensitive(true);
      max_br144->set_sensitive(false);
      max_br160->set_sensitive(true);
      max_br192->set_sensitive(true);
      max_br224->set_sensitive(true);
      max_br256->set_sensitive(true);
      max_br320->set_sensitive(true);

      break;

    case(16000):
    case(22050):
    case(24000):
      if (min_bitrate >= 192) {
        min_bitrate = 160;
        min_bitrate_optionmenu->set_history(13);
      }

      if (max_bitrate >= 192) {
        max_bitrate = 160;
        max_bitrate_optionmenu->set_history(13);
      }

      min_br8->set_sensitive(true);
      min_br16->set_sensitive(true);
      min_br24->set_sensitive(true);
      min_br32->set_sensitive(true);
      min_br40->set_sensitive(true);
      min_br48->set_sensitive(true);
      min_br56->set_sensitive(true);
      min_br64->set_sensitive(true);
      min_br80->set_sensitive(true);
      min_br96->set_sensitive(true);
      min_br112->set_sensitive(true);
      min_br128->set_sensitive(true);
      min_br144->set_sensitive(true);
      min_br160->set_sensitive(true);
      min_br192->set_sensitive(false);
      min_br224->set_sensitive(false);
      min_br256->set_sensitive(false);
      min_br320->set_sensitive(false);

      max_br8->set_sensitive(true);
      max_br16->set_sensitive(true);
      max_br24->set_sensitive(true);
      max_br32->set_sensitive(true);
      max_br40->set_sensitive(true);
      max_br48->set_sensitive(true);
      max_br56->set_sensitive(true);
      max_br64->set_sensitive(true);
      max_br80->set_sensitive(true);
      max_br96->set_sensitive(true);
      max_br112->set_sensitive(true);
      max_br128->set_sensitive(true);
      max_br144->set_sensitive(true);
      max_br160->set_sensitive(true);
      max_br192->set_sensitive(false);
      max_br224->set_sensitive(false);
      max_br256->set_sensitive(false);
      max_br320->set_sensitive(false);

      break;
  }
}

void DialogVBRoptions::button_pressed() {
  if (min_bitrate > max_bitrate) {
    Gnome::Dialog *dg = Gnome::Dialogs::warning(_("Minimum bitrate is higher than maximum bitrate!"));
    dg->run_and_close();
  }
  else {
    hide();
  }
}

int DialogVBRoptions::delete_event_impl(GdkEventAny* e) {
  hide();

  return true;            // disable destroying window by WM
}

vector<string> DialogVBRoptions::get_vbr_options() {
  vector<string> options;

  // disable Xing VBR informatial tag
  if (vbr_info->get_active()) options.push_back("-t");

  // minimum bitrate
  if (min_br->get_active()) {
    options.push_back("-b");

    strstream ss;
    string tmp;

    ss << min_bitrate;
    ss >> tmp;

    options.push_back(tmp);
  }

  // maximum bitrate
  if (max_br->get_active()) {
    options.push_back("-B");

    strstream ss;
    string tmp;

    ss << max_bitrate;
    ss >> tmp;

    options.push_back(tmp);
  }

  // set quality if it isn't default value
  if (quality != 4) {
   options.push_back("-V");

   strstream ss;
   string tmp;

   ss << quality;
   ss >> tmp;

   options.push_back(tmp);
  }

#ifdef DEBUG_OUTPUT
  cerr << "VBR options: ";
  copy(options.begin(), options.end(), ostream_iterator<string>(cerr, " "));
  cerr << endl;
#endif

  return options;
}
