/***************************************************************************
                mp3block.h  -  implementation of MPEG sound block
                             -------------------
    begin                : Sun Feb 13 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MP3BLOCK_H
#define MP3BLOCK_H

#include "soundblock.h"
#include "subprocess.h"
#include "dialogprogress.h"

#include <sigc++/signal_system.h>
#include <gtk--/main.h>

/**Block of sound - format MP3
  *@author Ladislav Slezk
  */

class MP3block : public SoundBlock {

  public:

  	MP3block();
  	MP3block(const MP3block &src);
  	~MP3block();

  	virtual bool init_play();  	
  	virtual bool play_block(SoundDevice *sd, string mpg123_path, int from_ms = -1, int to_ms = -1);
  	virtual bool is_changed();
  	virtual bool scan_file(int, int);					
  	virtual int block_length();								
  	virtual bool valid();											
  	virtual string block_info();							
  	virtual bool assign_file(string);
  	virtual bool assigned_file();							
  	virtual int num_samples();									
  	virtual int max_samples();									
	  virtual int read_sample(int pos, int channel = 0);
  	virtual void stop_block();
  	virtual void pause_block();
  	virtual void unpause_block();
  	virtual string get_type();
		virtual bool save_block_to_stream(ofstream &, string);
    virtual bool load_block_from_stream(ifstream &);
  	virtual int get_begin_sample_offset();
  	virtual int get_end_sample_offset();
  	virtual void set_begin_sample_offset(int);
  	virtual void set_end_sample_offset(int);

  	virtual void export_WAV(ofstream &ostr, int chan, int freq, int bps, DialogProgress *dp, string decoder_path, string conversion_path);
  	
		virtual void export_MP3(ofstream &ostr, string encoder_name, string encoder_path, int bitrate,
		                     int frequency, ChannelMode channel_mode, bool crc, bool priv, bool original,
		                     bool copyright, vector<string> VBR_options, vector<string> encoder_options,
		                     bool always_process, bool strict_equal, DialogProgress *dp);
  	
  	virtual int samples_per_frame();
  	virtual int get_begin_frame();
  	virtual int get_end_frame();
  	

		// static function - get info from MPEG header
  	static bool hdr_sync(guint32);
  	static int hdr_mpeg_version(guint32);
  	static int hdr_layer(guint32);
  	static int hdr_protection(guint32);
  	static int hdr_bitrate(guint32);
  	static int hdr_frequency(guint32);
  	static int hdr_padding(guint32);
  	static int hdr_private(guint32);
  	static int hdr_channels(guint32);
  	static MP3block::ChannelMode hdr_channel_mode(guint32);
  	static int hdr_mode_extension(guint32);
  	static int hdr_copyright(guint32);
  	static int hdr_original(guint32);
  	static int hdr_emphasis(guint32);
  	static int hdr_frame_size(guint32);
  	
  	// is ID3 tag present in file?
  	static bool has_id3v1_tag(ifstream &);  	
  	
  private:

		gint16 bitrate;                    // MPEG bitrate
		gint16 layer;                      // MPEG layer
		gint16 MPEGversion;                // MPEG version
		gint32 framesize;                  // size of frame - for estimating file length
		bool vbr;
		gint16 max_vbr;
		bool crc;
		bool private_flag;
		bool original;
		bool copyright;
		ChannelMode stereo_mode;
		
		SubProcess proc;                 // decoder process
		DialogProgress *progress;
		ofstream *export_stream;
    int block_num;
		DialogProgress *export_progress;
		int play_from_offs_ms;
		int finish;
    bool exporting_WAV;

		char *audiobuffer;              // MPEG buffer
		char *pausebuffer;              // decoded buffer
		int sizeofpausebuffer;          // size
		int usedBuffer;                 // size of used buffer
		
		bool scanning;                  // scanning is in progress
		bool exporting;                 // exporting is in progress
		int downSample;                 // downsample parametr
		int num;
		int beginScan;
		int endScan;
		double rat;
		int totalReceived;
		
		vector<gint32> frameOffs;
		void scanOffs();
		int audiobuffer_size;
    int playedMS;
    int tot_samples;

		void scanning_canceled();

    // slots for SubProcess	
    void player_exited(int);           // decoder exited
    void data_written();                 //
    void received_data(char *, int);   // received data from stdout
    void received_err(char *, int);    // received data from stderr
  	
};

#endif
