/***************************************************************************
              sounddevice.h  -  base class for all sound devices
             ----------------------------------------------------
    begin                : Tue Feb 1 2000
    copyright            : (C) 2000 by Ladislav Slezk
    email                : slezak@penguin.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SOUNDDEVICE_H
#define SOUNDDEVICE_H

#include <string>

using namespace std;

/**Base class for sound interface
  *@author Ladislav Slezk
  */

class SoundDevice {

  public:

  	SoundDevice();
  	virtual ~SoundDevice();

    typedef enum {bit8, bit16_le, bit16_be} bits;
  	
  	/** Open sound device
  	*/
    virtual bool open_device() = 0;

  	/** Returns true if device is open
  	*/
    virtual bool opened() = 0;

  	/** Close sound device
  	*/
    virtual bool close_device() = 0;

  	/** Set sampling frequency
		* @param freq Frequency in Hertz
  	*/
    virtual bool set_frequency(int freq) = 0;

  	/** Returns actual sampling frequency
  	*/
    virtual int get_frequency() = 0;

  	/** Set number of sound channels
    * @param ch Number of channels (1 = mono, 2 = stereo)
  	*/
    virtual bool set_number_of_channels(int ch) = 0;

  	/** Returns number of sound channels
  	*/
    virtual int get_number_of_channels() = 0;

  	/** Play data
  	*/
    virtual bool play(char *, int) = 0;

  	/** Stop playing
  	*/
    virtual bool stop() = 0;

  	/** Returns device info string
  	*/
    virtual string get_info() = 0;

  	/** Set output format
  	*/
    virtual bool set_format(bits) = 0;

  	/** Returns output format
  	*/
    virtual bits get_format() = 0;

  	/** Reset device
  	*/
    virtual void reset() = 0;

    virtual void set_device(string dev) = 0;

    virtual int get_processed_bytes() = 0;

    virtual int get_processed_ms() = 0;

  	/**
		* Returns fragment size - recommended size of data block
  	*/
    int get_fragment_size() {return fragment_size;}

   protected:

   	int fragment_size;
};

#endif
